import copy
from typing import Optional

from moto.stepfunctions.parser.api import (
    EvaluationFailedEventDetails,
    HistoryEventType,
    MapStateStartedEventDetails,
)
from moto.stepfunctions.parser.asl.component.common.catch.catch_decl import CatchDecl
from moto.stepfunctions.parser.asl.component.common.catch.catch_outcome import (
    CatchOutcome,
)
from moto.stepfunctions.parser.asl.component.common.error_name.failure_event import (
    FailureEvent,
    FailureEventException,
)
from moto.stepfunctions.parser.asl.component.common.error_name.states_error_name import (
    StatesErrorName,
)
from moto.stepfunctions.parser.asl.component.common.error_name.states_error_name_type import (
    StatesErrorNameType,
)
from moto.stepfunctions.parser.asl.component.common.parargs import Parameters, Parargs
from moto.stepfunctions.parser.asl.component.common.path.items_path import ItemsPath
from moto.stepfunctions.parser.asl.component.common.path.result_path import ResultPath
from moto.stepfunctions.parser.asl.component.common.result_selector import (
    ResultSelector,
)
from moto.stepfunctions.parser.asl.component.common.retry.retry_decl import RetryDecl
from moto.stepfunctions.parser.asl.component.common.retry.retry_outcome import (
    RetryOutcome,
)
from moto.stepfunctions.parser.asl.component.state.exec.execute_state import (
    ExecutionState,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.item_reader.item_reader_decl import (
    ItemReader,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.item_selector import (
    ItemSelector,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.items.items import (
    Items,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.itemprocessor.distributed_item_processor import (
    DistributedItemProcessor,
    DistributedItemProcessorEvalInput,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.itemprocessor.inline_item_processor import (
    InlineItemProcessor,
    InlineItemProcessorEvalInput,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.itemprocessor.item_processor_decl import (
    ItemProcessorDecl,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.itemprocessor.item_processor_factory import (
    from_item_processor_decl,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.iteration_component import (
    IterationComponent,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.iterator.distributed_iterator import (
    DistributedIterator,
    DistributedIteratorEvalInput,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.iterator.inline_iterator import (
    InlineIterator,
    InlineIteratorEvalInput,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.iterator.iterator_decl import (
    IteratorDecl,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.iteration.iterator.iterator_factory import (
    from_iterator_decl,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.label import (
    Label,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.max_concurrency import (
    MaxConcurrency,
    MaxConcurrencyDecl,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.result_writer.result_writer_decl import (
    ResultWriter,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_map.tolerated_failure import (
    ToleratedFailureCount,
    ToleratedFailureCountDecl,
    ToleratedFailurePercentage,
    ToleratedFailurePercentageDecl,
)
from moto.stepfunctions.parser.asl.component.state.state_props import StateProps
from moto.stepfunctions.parser.asl.eval.environment import Environment
from moto.stepfunctions.parser.asl.eval.event.event_detail import EventDetails


class StateMap(ExecutionState):
    items: Optional[Items]
    items_path: Optional[ItemsPath]
    iteration_component: IterationComponent
    item_reader: Optional[ItemReader]
    item_selector: Optional[ItemSelector]
    parameters: Optional[Parameters]
    max_concurrency_decl: MaxConcurrencyDecl
    tolerated_failure_count_decl: ToleratedFailureCountDecl
    tolerated_failure_percentage_decl: ToleratedFailurePercentage
    result_path: Optional[ResultPath]
    result_selector: ResultSelector
    retry: Optional[RetryDecl]
    catch: Optional[CatchDecl]
    label: Optional[Label]
    result_writer: Optional[ResultWriter]

    def __init__(self):
        super(StateMap, self).__init__(
            state_entered_event_type=HistoryEventType.MapStateEntered,
            state_exited_event_type=HistoryEventType.MapStateExited,
        )

    def from_state_props(self, state_props: StateProps) -> None:
        super(StateMap, self).from_state_props(state_props)
        if self._is_language_query_jsonpath():
            self.items = None
            self.items_path = state_props.get(ItemsPath) or ItemsPath()
        else:
            # TODO: add snapshot test to assert what missing definitions of items means for a states map
            self.items_path = None
            self.items = state_props.get(Items)
        self.item_reader = state_props.get(ItemReader)
        self.item_selector = state_props.get(ItemSelector)
        self.parameters = state_props.get(Parargs)
        self.max_concurrency_decl = (
            state_props.get(MaxConcurrencyDecl) or MaxConcurrency()
        )
        self.tolerated_failure_count_decl = (
            state_props.get(ToleratedFailureCountDecl) or ToleratedFailureCount()
        )
        self.tolerated_failure_percentage_decl = (
            state_props.get(ToleratedFailurePercentageDecl)
            or ToleratedFailurePercentage()
        )
        self.result_path = state_props.get(ResultPath) or ResultPath(
            result_path_src=ResultPath.DEFAULT_PATH
        )
        self.result_selector = state_props.get(ResultSelector)
        self.retry = state_props.get(RetryDecl)
        self.catch = state_props.get(CatchDecl)
        self.label = state_props.get(Label)
        self.result_writer = state_props.get(ResultWriter)

        iterator_decl = state_props.get(typ=IteratorDecl)
        item_processor_decl = state_props.get(typ=ItemProcessorDecl)

        if iterator_decl and item_processor_decl:
            raise ValueError("Cannot define both Iterator and ItemProcessor.")

        iteration_decl = iterator_decl or item_processor_decl
        if iteration_decl is None:
            raise ValueError(
                f"Missing ItemProcessor/Iterator definition in props '{state_props}'."
            )

        if isinstance(iteration_decl, IteratorDecl):
            self.iteration_component = from_iterator_decl(iteration_decl)
        elif isinstance(iteration_decl, ItemProcessorDecl):
            self.iteration_component = from_item_processor_decl(iteration_decl)
        else:
            raise ValueError(f"Unknown value for IteratorDecl '{iteration_decl}'.")

    def _eval_execution(self, env: Environment) -> None:
        self.max_concurrency_decl.eval(env=env)
        max_concurrency_num = env.stack.pop()
        label = self.label.label if self.label else None

        # Despite MaxConcurrency and Tolerance fields being state level fields, AWS StepFunctions evaluates only
        # MaxConcurrency as a state level field. In contrast, Tolerance is evaluated only after the state start
        # event but is logged with event IDs coherent with state level fields. To adhere to this quirk, an evaluation
        # frame from this point is created for the evaluation of Tolerance fields following the state start event.
        frame: Environment = env.open_frame()
        frame.states.reset(input_value=env.states.get_input())
        frame.stack = copy.deepcopy(env.stack)

        try:
            if self.items_path:
                self.items_path.eval(env=env)

            if self.items:
                self.items.eval(env=env)

            if self.item_reader:
                env.event_manager.add_event(
                    context=env.event_history_context,
                    event_type=HistoryEventType.MapStateStarted,
                    event_details=EventDetails(
                        mapStateStartedEventDetails=MapStateStartedEventDetails(
                            length=0
                        )
                    ),
                )
                input_items = None
            else:
                input_items = env.stack.pop()
                # TODO: This should probably be raised within an Items EvalComponent
                if not isinstance(input_items, list):
                    error_name = StatesErrorName(
                        typ=StatesErrorNameType.StatesQueryEvaluationError
                    )
                    failure_event = FailureEvent(
                        env=env,
                        error_name=error_name,
                        event_type=HistoryEventType.EvaluationFailed,
                        event_details=EventDetails(
                            evaluationFailedEventDetails=EvaluationFailedEventDetails(
                                cause=f"Map state input must be an array but was: {type(input_items)}",
                                error=error_name.error_name,
                            )
                        ),
                    )
                    raise FailureEventException(failure_event=failure_event)
                env.event_manager.add_event(
                    context=env.event_history_context,
                    event_type=HistoryEventType.MapStateStarted,
                    event_details=EventDetails(
                        mapStateStartedEventDetails=MapStateStartedEventDetails(
                            length=len(input_items)
                        )
                    ),
                )

            self.tolerated_failure_count_decl.eval(env=frame)
            tolerated_failure_count = frame.stack.pop()
            self.tolerated_failure_percentage_decl.eval(env=frame)
            tolerated_failure_percentage = frame.stack.pop()
        finally:
            env.close_frame(frame)

        if isinstance(self.iteration_component, InlineIterator):
            eval_input = InlineIteratorEvalInput(
                state_name=self.name,
                max_concurrency=max_concurrency_num,
                input_items=input_items,
                parameters=self.parameters,
                item_selector=self.item_selector,
            )
        elif isinstance(self.iteration_component, DistributedIterator):
            eval_input = DistributedIteratorEvalInput(
                state_name=self.name,
                max_concurrency=max_concurrency_num,
                input_items=input_items,
                parameters=self.parameters,
                item_selector=self.item_selector,
                item_reader=self.item_reader,
                tolerated_failure_count=tolerated_failure_count,
                tolerated_failure_percentage=tolerated_failure_percentage,
                label=label,
            )
        elif isinstance(self.iteration_component, InlineItemProcessor):
            eval_input = InlineItemProcessorEvalInput(
                state_name=self.name,
                max_concurrency=max_concurrency_num,
                input_items=input_items,
                item_selector=self.item_selector,
                parameters=self.parameters,
            )
        elif isinstance(self.iteration_component, DistributedItemProcessor):
            eval_input = DistributedItemProcessorEvalInput(
                state_name=self.name,
                max_concurrency=max_concurrency_num,
                input_items=input_items,
                item_reader=self.item_reader,
                item_selector=self.item_selector,
                parameters=self.parameters,
                tolerated_failure_count=tolerated_failure_count,
                tolerated_failure_percentage=tolerated_failure_percentage,
                label=label,
            )
        else:
            raise RuntimeError(
                f"Unknown iteration component of type '{type(self.iteration_component)}' '{self.iteration_component}'."
            )

        env.stack.append(eval_input)
        self.iteration_component.eval(env)

        if self.result_writer:
            self.result_writer.eval(env)

        env.event_manager.add_event(
            context=env.event_history_context,
            event_type=HistoryEventType.MapStateSucceeded,
            update_source_event_id=False,
        )

    def _eval_state(self, env: Environment) -> None:
        # Initialise the retry counter for execution states.
        env.states.context_object.context_object_data["State"]["RetryCount"] = 0

        # Attempt to evaluate the state's logic through until it's successful, caught, or retries have run out.
        while env.is_running():
            try:
                self._evaluate_with_timeout(env)
                break
            except Exception as ex:
                failure_event: FailureEvent = self._from_error(env=env, ex=ex)
                error_output = self._construct_error_output_value(
                    failure_event=failure_event
                )
                env.states.set_error_output(error_output)
                env.states.set_result(error_output)

                if self.retry:
                    retry_outcome: RetryOutcome = self._handle_retry(
                        env=env, failure_event=failure_event
                    )
                    if retry_outcome == RetryOutcome.CanRetry:
                        continue

                if failure_event.event_type != HistoryEventType.ExecutionFailed:
                    if (
                        isinstance(ex, FailureEventException)
                        and failure_event.event_type
                        == HistoryEventType.EvaluationFailed
                    ):
                        env.event_manager.add_event(
                            context=env.event_history_context,
                            event_type=HistoryEventType.EvaluationFailed,
                            event_details=EventDetails(
                                evaluationFailedEventDetails=ex.get_evaluation_failed_event_details(),
                            ),
                        )
                    env.event_manager.add_event(
                        context=env.event_history_context,
                        event_type=HistoryEventType.MapStateFailed,
                    )

                if self.catch:
                    self._handle_catch(env=env, failure_event=failure_event)
                    catch_outcome: CatchOutcome = env.stack[-1]
                    if catch_outcome == CatchOutcome.Caught:
                        break

                self._handle_uncaught(env=env, failure_event=failure_event)



"""osis base URL and path."""

from .responses import OpenSearchIngestionResponse

url_bases = [
    r"https?://osis\.(.+)\.amazonaws\.com",
]

url_paths = {
    "{0}/2022-01-01/osis/createPipeline$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/deletePipeline/(?P<PipelineName>[^/]+)$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/getPipeline/(?P<PipelineName>[^/]+)$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/startPipeline/(?P<PipelineName>[^/]+)$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/stopPipeline/(?P<PipelineName>[^/]+)$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/listPipelines$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/listTagsForResource/$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/updatePipeline/(?P<PipelineName>[^/]+)$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/tagResource/$": OpenSearchIngestionResponse.dispatch,
    "{0}/2022-01-01/osis/untagResource/$": OpenSearchIngestionResponse.dispatch,
}



import boto3
import pytest
from botocore.exceptions import ClientError
from freezegun import freeze_time

from moto import mock_aws, settings

# See our Development Tips on writing tests for hints on how to write good tests:
# http://docs.getmoto.org/en/latest/docs/contributing/development_tips/tests.html


@mock_aws
def test_create_domain__minimal_options():
    client = boto3.client("opensearch", region_name="eu-west-1")
    status = client.create_domain(DomainName="testdn")["DomainStatus"]
    assert "DomainId" in status
    assert "DomainName" in status
    assert status["DomainName"] == "testdn"
    assert status["Endpoint"] is not None
    assert "Endpoints" not in status


@mock_aws
def test_create_domain_in_vpc():
    client = boto3.client("opensearch", region_name="eu-west-1")
    status = client.create_domain(
        DomainName="testdn", VPCOptions={"SubnetIds": ["sub1"]}
    )["DomainStatus"]
    assert "DomainId" in status
    assert "DomainName" in status
    assert status["DomainName"] == "testdn"
    assert "Endpoint" not in status
    assert status["Endpoints"] is not None


@mock_aws
def test_create_domain_with_some_options():
    client = boto3.client("opensearch", region_name="eu-north-1")
    status = client.create_domain(
        DomainName="testdn",
        DomainEndpointOptions={
            "CustomEndpointEnabled": False,
            "EnforceHTTPS": True,
            "TLSSecurityPolicy": "Policy-Min-TLS-1-0-2019-07",
        },
        EBSOptions={"EBSEnabled": True, "VolumeSize": 10},
        SnapshotOptions={"AutomatedSnapshotStartHour": 20},
        EngineVersion="OpenSearch_1.1",
    )["DomainStatus"]
    assert status["Created"]
    assert status["EngineVersion"]["Options"] == "OpenSearch_1.1"
    assert status["DomainEndpointOptions"] == {
        "EnforceHTTPS": True,
        "TLSSecurityPolicy": "Policy-Min-TLS-1-0-2019-07",
        "CustomEndpointEnabled": False,
    }
    assert status["EBSOptions"] == {"EBSEnabled": True, "VolumeSize": 10}
    assert status["SnapshotOptions"] == {"AutomatedSnapshotStartHour": 20}


@mock_aws
def test_get_compatible_versions():
    client = boto3.client("opensearch", region_name="us-east-2")
    client.create_domain(DomainName="testdn")

    versions = client.get_compatible_versions(DomainName="testdn")["CompatibleVersions"]
    assert len(versions) == 22


@mock_aws
def test_get_compatible_versions_unknown_domain():
    client = boto3.client("opensearch", region_name="us-east-2")

    with pytest.raises(ClientError) as exc:
        client.get_compatible_versions(DomainName="testdn")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"
    assert err["Message"] == "Domain not found: testdn"


@mock_aws
def test_describe_unknown_domain():
    client = boto3.client("opensearch", region_name="eu-west-1")

    with pytest.raises(ClientError) as exc:
        client.describe_domain(DomainName="testdn")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"
    assert err["Message"] == "Domain not found: testdn"


@mock_aws
@freeze_time("2015-01-01 12:00:00")
def test_describe_domain():
    # Setup
    client = boto3.client("opensearch", region_name="eu-west-1")
    client.create_domain(DomainName="testdn")

    # Execute
    status = client.describe_domain(DomainName="testdn")["DomainStatus"]
    ev_status = status["EngineVersion"]["Status"]

    # Verify
    assert "DomainId" in status
    assert "DomainName" in status
    assert status["DomainName"] == "testdn"
    assert status["EngineVersion"]["Options"] == "OpenSearch_2.5"
    assert ev_status["State"] == "Active"
    assert ev_status["PendingDeletion"] is False

    if not settings.TEST_SERVER_MODE:
        assert ev_status["CreationDate"] == 1420113600.0
        assert ev_status["UpdateDate"] == 1420113600.0


@mock_aws
def test_delete_domain():
    client = boto3.client("opensearch", region_name="eu-west-1")
    client.create_domain(DomainName="testdn")
    resp = client.delete_domain(DomainName="testdn")
    status = resp["DomainStatus"]
    assert "DomainId" in status
    assert "DomainName" in status
    assert status["Deleted"]  # assume deleted completion
    assert status["DomainName"] == "testdn"


@mock_aws
def test_delete_invalid_domain():
    client = boto3.client("opensearch", region_name="eu-west-1")
    with pytest.raises(ClientError) as exc:
        client.describe_domain(DomainName="testdn")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"
    assert err["Message"] == "Domain not found: testdn"


@mock_aws
def test_update_domain_config():
    client = boto3.client("opensearch", region_name="eu-north-1")
    client.create_domain(
        DomainName="testdn",
        DomainEndpointOptions={
            "CustomEndpointEnabled": False,
            "EnforceHTTPS": True,
            "TLSSecurityPolicy": "Policy-Min-TLS-1-0-2019-07",
        },
        EBSOptions={"EBSEnabled": True, "VolumeSize": 10},
        EngineVersion="OpenSearch 1.1",
    )

    config = client.update_domain_config(
        DomainName="testdn",
        EBSOptions={"EBSEnabled": False},
    )["DomainConfig"]

    assert config["EBSOptions"] == {"Options": {"EBSEnabled": False}}
    assert config["DomainEndpointOptions"] == {
        "Options": {
            "EnforceHTTPS": True,
            "TLSSecurityPolicy": "Policy-Min-TLS-1-0-2019-07",
            "CustomEndpointEnabled": False,
        }
    }


@mock_aws
def test_list_domain_names():
    client = boto3.client("opensearch", region_name="ap-southeast-1")

    test_domain_names_list_exist = False

    opensearch_domain_name = "testdn"
    opensearch_engine_version = "OpenSearch_1.0"
    client.create_domain(
        DomainName=opensearch_domain_name, EngineVersion=opensearch_engine_version
    )

    resp = client.list_domain_names()
    domain_names = resp["DomainNames"]

    for domain_name in domain_names:
        if domain_name["DomainName"] == opensearch_domain_name:
            test_domain_names_list_exist = True

    assert resp["ResponseMetadata"]["HTTPStatusCode"] == 200
    assert test_domain_names_list_exist


@mock_aws
def test_list_filtered_domain_names():
    client = boto3.client("opensearch", region_name="ap-southeast-1")

    test_domain_names_list_exist = False

    opensearch_domain_name = "testdn"
    opensearch_engine_version = "OpenSearch_1.0"
    client.create_domain(
        DomainName=opensearch_domain_name, EngineVersion=opensearch_engine_version
    )

    resp = client.list_domain_names(EngineType="OpenSearch")
    domain_names = resp["DomainNames"]

    for domain_name in domain_names:
        if domain_name["DomainName"] == opensearch_domain_name:
            if domain_name["EngineType"] == opensearch_engine_version.split("_")[0]:
                test_domain_names_list_exist = True

    assert resp["ResponseMetadata"]["HTTPStatusCode"] == 200
    assert test_domain_names_list_exist


@mock_aws
def test_list_unknown_domain_names_engine_type():
    client = boto3.client("opensearch", region_name="ap-southeast-1")

    opensearch_domain_name = "testdn"
    opensearch_engine_version = "OpenSearch_1.0"
    client.create_domain(
        DomainName=opensearch_domain_name, EngineVersion=opensearch_engine_version
    )

    with pytest.raises(ClientError) as exc:
        client.list_domain_names(EngineType="unknown")
    err = exc.value.response["Error"]
    assert err["Code"] == "EngineTypeNotFoundException"
    assert err["Message"] == "Engine Type not found: unknown"


@mock_aws
def test_describe_domains():
    client = boto3.client("opensearch", region_name="us-east-1")
    domain_names = [f"env{i}" for i in range(1, 5)]
    opensearch_engine_version = "OpenSearch_1.0"
    for name in domain_names:
        client.create_domain(DomainName=name, EngineVersion=opensearch_engine_version)
    resp = client.describe_domains(DomainNames=domain_names)

    assert len(resp["DomainStatusList"]) == 4
    for domain in resp["DomainStatusList"]:
        assert domain["DomainName"] in domain_names
        assert "AdvancedSecurityOptions" in domain.keys()
        assert "AdvancedOptions" in domain.keys()

    # Test for invalid domain name
    resp = client.describe_domains(DomainNames=["invalid"])
    assert len(resp["DomainStatusList"]) == 0



from __future__ import annotations

import abc
from typing import Any

from moto.stepfunctions.parser.asl.component.common.assign.assign_template_value import (
    AssignTemplateValue,
)
from moto.stepfunctions.parser.asl.component.common.variable_sample import (
    VariableSample,
)
from moto.stepfunctions.parser.asl.component.eval_component import EvalComponent
from moto.stepfunctions.parser.asl.component.intrinsic.function.function import Function
from moto.stepfunctions.parser.asl.eval.environment import Environment
from moto.stepfunctions.parser.asl.parse.intrinsic.intrinsic_parser import (
    IntrinsicParser,
)
from moto.stepfunctions.parser.asl.utils.json_path import extract_json


class AssignTemplateBinding(EvalComponent, abc.ABC):
    identifier: str

    def __init__(self, identifier: str):
        super().__init__()
        self.identifier = identifier

    @abc.abstractmethod
    def _eval_value(self, env: Environment) -> Any: ...

    def _eval_body(self, env: Environment) -> None:
        assign_object: dict = env.stack.pop()
        assign_value = self._eval_value(env=env)
        assign_object[self.identifier] = assign_value
        env.stack.append(assign_object)


class AssignTemplateBindingPath(AssignTemplateBinding):
    path: str

    def __init__(self, identifier: str, path: str):
        super().__init__(identifier=identifier)
        self.path = path

    def _eval_value(self, env: Environment) -> Any:
        memory_value = env.stack[-1]
        path_output = extract_json(self.path, memory_value)
        return path_output


class AssignTemplateBindingPathContext(AssignTemplateBindingPath):
    @classmethod
    def from_raw(
        cls, identifier: str, string_path_context_obj: str
    ) -> AssignTemplateBindingPathContext:
        path_context_obj: str = string_path_context_obj[1:]
        return cls(identifier=identifier, path=path_context_obj)

    def _eval_value(self, env: Environment) -> Any:
        path_output = extract_json(
            self.path, env.states.context_object.context_object_data
        )
        return path_output


class AssignTemplateBindingIntrinsicFunction(AssignTemplateBinding):
    function_literal: str
    function: Function

    def __init__(self, identifier: str, function_literal: str):
        super().__init__(identifier=identifier)
        self.function_literal = function_literal
        self.function, _ = IntrinsicParser.parse(self.function_literal)

    def _eval_value(self, env: Environment) -> Any:
        # TODO: resolve jsonata variable references as arguments.
        #  should probably be done in the function object.
        self.function.eval(env=env)
        val = env.stack.pop()
        return val


class AssignTemplateBindingVar(AssignTemplateBinding):
    variable_sample: VariableSample

    def __init__(self, identifier: str, variable_sample: VariableSample):
        super().__init__(identifier=identifier)
        self.variable_sample = variable_sample

    def _eval_value(self, env: Environment) -> Any:
        self.variable_sample.eval(env=env)
        value = env.stack.pop()
        return value


class AssignTemplateBindingValue(AssignTemplateBinding):
    assign_value: AssignTemplateValue

    def __init__(self, identifier: str, assign_value: AssignTemplateValue):
        super().__init__(identifier=identifier)
        self.assign_value = assign_value

    def _eval_value(self, env: Environment) -> Any:
        self.assign_value.eval(env=env)
        value = env.stack.pop()
        return value



"""Handles incoming directconnect requests, invokes methods, returns responses."""

import json

from moto.core.responses import BaseResponse

from .models import LAG, Connection, DirectConnectBackend, directconnect_backends


class DirectConnectResponse(BaseResponse):
    """Handler for DirectConnect requests and responses."""

    def __init__(self) -> None:
        super().__init__(service_name="directconnect")

    @property
    def directconnect_backend(self) -> DirectConnectBackend:
        return directconnect_backends[self.current_account][self.region]

    def describe_connections(self) -> str:
        params = json.loads(self.body)
        connections = self.directconnect_backend.describe_connections(
            connection_id=params.get("connectionId"),
        )
        return json.dumps(
            dict(connections=[connection.to_dict() for connection in connections])
        )

    def create_connection(self) -> str:
        params = json.loads(self.body)
        connection: Connection = self.directconnect_backend.create_connection(
            location=params.get("location"),
            bandwidth=params.get("bandwidth"),
            connection_name=params.get("connectionName"),
            lag_id=params.get("lagId"),
            tags=params.get("tags"),
            provider_name=params.get("providerName"),
            request_mac_sec=params.get("requestMACSec"),
        )
        return json.dumps(connection.to_dict())

    def delete_connection(self) -> str:
        params = json.loads(self.body)
        connection: Connection = self.directconnect_backend.delete_connection(
            connection_id=params.get("connectionId"),
        )
        return json.dumps(connection.to_dict())

    def update_connection(self) -> str:
        params = json.loads(self.body)
        connection: Connection = self.directconnect_backend.update_connection(
            connection_id=params.get("connectionId"),
            new_connection_name=params.get("connectionName"),
            new_encryption_mode=params.get("encryptionMode"),
        )
        return json.dumps(connection.to_dict())

    def associate_mac_sec_key(self) -> str:
        params = json.loads(self.body)
        connection_id = params.get("connectionId")
        secret_arn = params.get("secretARN")
        ckn = params.get("ckn")
        cak = params.get("cak")
        connection_id, mac_sec_keys = self.directconnect_backend.associate_mac_sec_key(
            connection_id=connection_id,
            secret_arn=secret_arn,
            ckn=ckn,
            cak=cak,
        )
        return json.dumps(
            dict(
                connectionId=connection_id,
                macSecKeys=[mac_sec_key.to_dict() for mac_sec_key in mac_sec_keys],
            )
        )

    def create_lag(self) -> str:
        params = json.loads(self.body)
        number_of_connections = params.get("numberOfConnections")
        location = params.get("location")
        connections_bandwidth = params.get("connectionsBandwidth")
        lag_name = params.get("lagName")
        connection_id = params.get("connectionId")
        tags = params.get("tags")
        child_connection_tags = params.get("childConnectionTags")
        provider_name = params.get("providerName")
        request_mac_sec = params.get("requestMACSec")
        lag: LAG = self.directconnect_backend.create_lag(
            number_of_connections=number_of_connections,
            location=location,
            connections_bandwidth=connections_bandwidth,
            lag_name=lag_name,
            connection_id=connection_id,
            tags=tags,
            child_connection_tags=child_connection_tags,
            provider_name=provider_name,
            request_mac_sec=request_mac_sec,
        )
        return json.dumps(lag.to_dict())

    def describe_lags(self) -> str:
        params = json.loads(self.body)
        lags = self.directconnect_backend.describe_lags(
            lag_id=params.get("lagId"),
        )
        return json.dumps(dict(lags=[lag.to_dict() for lag in lags]))

    def disassociate_mac_sec_key(self) -> str:
        params = json.loads(self.body)
        connection_id = params.get("connectionId")
        secret_arn = params.get("secretARN")
        connection_id, mac_sec_key = (
            self.directconnect_backend.disassociate_mac_sec_key(
                connection_id=connection_id,
                secret_arn=secret_arn,
            )
        )
        return json.dumps(
            dict(
                connectionId=connection_id,
                macSecKeys=[mac_sec_key.to_dict()],
            )
        )



import boto3
import pytest
from botocore.exceptions import ClientError

from moto import mock_aws


@mock_aws
def test_put_gateway_response_minimal():
    client = boto3.client("apigateway", region_name="us-east-2")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]

    resp = client.put_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")

    assert resp["responseType"] == "DEFAULT_4XX"
    assert resp["defaultResponse"] is False


@mock_aws
def test_put_gateway_response():
    client = boto3.client("apigateway", region_name="us-east-2")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]

    resp = client.put_gateway_response(
        restApiId=api_id,
        responseType="DEFAULT_4XX",
        statusCode="401",
        responseParameters={"gatewayresponse.header.Authorization": "'Basic'"},
        responseTemplates={
            "application/xml": "#set($inputRoot = $input.path('$'))\n{ }"
        },
    )

    assert resp["responseType"] == "DEFAULT_4XX"
    assert resp["defaultResponse"] is False
    assert resp["statusCode"] == "401"
    assert resp["responseParameters"] == {
        "gatewayresponse.header.Authorization": "'Basic'"
    }
    assert resp["responseTemplates"] == {
        "application/xml": "#set($inputRoot = $input.path('$'))\n{ }"
    }


@mock_aws
def test_get_gateway_response_minimal():
    client = boto3.client("apigateway", region_name="ap-southeast-1")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]

    client.put_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")

    resp = client.get_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")

    assert resp["responseType"] == "DEFAULT_4XX"
    assert resp["defaultResponse"] is False


@mock_aws
def test_get_gateway_response():
    client = boto3.client("apigateway", region_name="us-east-2")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]

    client.put_gateway_response(
        restApiId=api_id,
        responseType="DEFAULT_4XX",
        statusCode="401",
        responseParameters={"gatewayresponse.header.Authorization": "'Basic'"},
        responseTemplates={
            "application/xml": "#set($inputRoot = $input.path('$'))\n{ }"
        },
    )

    resp = client.get_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")

    assert resp["responseType"] == "DEFAULT_4XX"
    assert resp["defaultResponse"] is False
    assert resp["statusCode"] == "401"
    assert resp["responseParameters"] == {
        "gatewayresponse.header.Authorization": "'Basic'"
    }
    assert resp["responseTemplates"] == {
        "application/xml": "#set($inputRoot = $input.path('$'))\n{ }"
    }


@mock_aws
def test_get_gateway_response_unknown():
    client = boto3.client("apigateway", region_name="us-east-2")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]

    with pytest.raises(ClientError) as exc:
        client.get_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")
    err = exc.value.response["Error"]
    assert err["Code"] == "NotFoundException"


@mock_aws
def test_get_gateway_responses_empty():
    client = boto3.client("apigateway", region_name="ap-southeast-1")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]
    resp = client.get_gateway_responses(restApiId=api_id)

    assert resp["items"] == []


@mock_aws
def test_get_gateway_responses():
    client = boto3.client("apigateway", region_name="ap-southeast-1")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]
    client.put_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")
    client.put_gateway_response(
        restApiId=api_id, responseType="DEFAULT_5XX", statusCode="503"
    )

    resp = client.get_gateway_responses(restApiId=api_id)
    assert len(resp["items"]) == 2

    assert {"responseType": "DEFAULT_4XX", "defaultResponse": False} in resp["items"]
    assert {
        "responseType": "DEFAULT_5XX",
        "defaultResponse": False,
        "statusCode": "503",
    } in resp["items"]


@mock_aws
def test_delete_gateway_response():
    client = boto3.client("apigateway", region_name="ap-southeast-1")
    api_id = client.create_rest_api(name="my_api", description="d")["id"]
    client.put_gateway_response(restApiId=api_id, responseType="DEFAULT_4XX")
    client.put_gateway_response(
        restApiId=api_id, responseType="DEFAULT_5XX", statusCode="503"
    )

    resp = client.get_gateway_responses(restApiId=api_id)
    assert len(resp["items"]) == 2

    client.delete_gateway_response(restApiId=api_id, responseType="DEFAULT_5XX")

    resp = client.get_gateway_responses(restApiId=api_id)
    assert len(resp["items"]) == 1
    assert {"responseType": "DEFAULT_4XX", "defaultResponse": False} in resp["items"]



from typing import List

from moto.stepfunctions.parser.asl.component.common.assign.assign_template_binding import (
    AssignTemplateBinding,
)
from moto.stepfunctions.parser.asl.component.common.assign.assign_template_value import (
    AssignTemplateValue,
)
from moto.stepfunctions.parser.asl.eval.environment import Environment


class AssignTemplateValueObject(AssignTemplateValue):
    bindings: List[AssignTemplateBinding]

    def __init__(self, bindings: List[AssignTemplateBinding]):
        self.bindings = bindings

    def _eval_body(self, env: Environment) -> None:
        env.stack.append(dict())
        for binding in self.bindings:
            binding.eval(env)



"""opensearch base URL and path."""

from .responses import OpenSearchServiceResponse

url_bases = [r"https?://es\.(.+)\.amazonaws\.com"]


response = OpenSearchServiceResponse()


url_paths = {
    "{0}/.*$": response.dispatch,
}



# Generated from ASLParser.g4 by ANTLR 4.13.2
from antlr4 import ParseTreeVisitor

if "." in __name__:
    from .ASLParser import ASLParser
else:
    from ASLParser import ASLParser

# This class defines a complete generic visitor for a parse tree produced by ASLParser.


class ASLParserVisitor(ParseTreeVisitor):
    # Visit a parse tree produced by ASLParser#state_machine.
    def visitState_machine(self, ctx: ASLParser.State_machineContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#program_decl.
    def visitProgram_decl(self, ctx: ASLParser.Program_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#top_layer_stmt.
    def visitTop_layer_stmt(self, ctx: ASLParser.Top_layer_stmtContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#startat_decl.
    def visitStartat_decl(self, ctx: ASLParser.Startat_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comment_decl.
    def visitComment_decl(self, ctx: ASLParser.Comment_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#version_decl.
    def visitVersion_decl(self, ctx: ASLParser.Version_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#query_language_decl.
    def visitQuery_language_decl(self, ctx: ASLParser.Query_language_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#state_stmt.
    def visitState_stmt(self, ctx: ASLParser.State_stmtContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#states_decl.
    def visitStates_decl(self, ctx: ASLParser.States_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#state_name.
    def visitState_name(self, ctx: ASLParser.State_nameContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#state_decl.
    def visitState_decl(self, ctx: ASLParser.State_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#state_decl_body.
    def visitState_decl_body(self, ctx: ASLParser.State_decl_bodyContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#type_decl.
    def visitType_decl(self, ctx: ASLParser.Type_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#next_decl.
    def visitNext_decl(self, ctx: ASLParser.Next_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#resource_decl.
    def visitResource_decl(self, ctx: ASLParser.Resource_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#input_path_decl_var.
    def visitInput_path_decl_var(self, ctx: ASLParser.Input_path_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#input_path_decl_path_context_object.
    def visitInput_path_decl_path_context_object(
        self, ctx: ASLParser.Input_path_decl_path_context_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#input_path_decl_path.
    def visitInput_path_decl_path(self, ctx: ASLParser.Input_path_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#result_decl.
    def visitResult_decl(self, ctx: ASLParser.Result_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#result_path_decl.
    def visitResult_path_decl(self, ctx: ASLParser.Result_path_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#output_path_decl_var.
    def visitOutput_path_decl_var(self, ctx: ASLParser.Output_path_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#output_path_decl_path_context_object.
    def visitOutput_path_decl_path_context_object(
        self, ctx: ASLParser.Output_path_decl_path_context_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#output_path_decl_path.
    def visitOutput_path_decl_path(self, ctx: ASLParser.Output_path_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#end_decl.
    def visitEnd_decl(self, ctx: ASLParser.End_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#default_decl.
    def visitDefault_decl(self, ctx: ASLParser.Default_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_jsonata.
    def visitError_jsonata(self, ctx: ASLParser.Error_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_string.
    def visitError_string(self, ctx: ASLParser.Error_stringContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_path_decl_var.
    def visitError_path_decl_var(self, ctx: ASLParser.Error_path_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_path_decl_path.
    def visitError_path_decl_path(self, ctx: ASLParser.Error_path_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_path_decl_context.
    def visitError_path_decl_context(
        self, ctx: ASLParser.Error_path_decl_contextContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_path_decl_intrinsic.
    def visitError_path_decl_intrinsic(
        self, ctx: ASLParser.Error_path_decl_intrinsicContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_jsonata.
    def visitCause_jsonata(self, ctx: ASLParser.Cause_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_string.
    def visitCause_string(self, ctx: ASLParser.Cause_stringContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_path_decl_var.
    def visitCause_path_decl_var(self, ctx: ASLParser.Cause_path_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_path_decl_path.
    def visitCause_path_decl_path(self, ctx: ASLParser.Cause_path_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_path_decl_context.
    def visitCause_path_decl_context(
        self, ctx: ASLParser.Cause_path_decl_contextContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#cause_path_decl_intrinsic.
    def visitCause_path_decl_intrinsic(
        self, ctx: ASLParser.Cause_path_decl_intrinsicContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#seconds_jsonata.
    def visitSeconds_jsonata(self, ctx: ASLParser.Seconds_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#seconds_int.
    def visitSeconds_int(self, ctx: ASLParser.Seconds_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#seconds_path_decl_var.
    def visitSeconds_path_decl_var(self, ctx: ASLParser.Seconds_path_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#seconds_path_decl_value.
    def visitSeconds_path_decl_value(
        self, ctx: ASLParser.Seconds_path_decl_valueContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timestamp_jsonata.
    def visitTimestamp_jsonata(self, ctx: ASLParser.Timestamp_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timestamp_string.
    def visitTimestamp_string(self, ctx: ASLParser.Timestamp_stringContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timestamp_path_decl_var.
    def visitTimestamp_path_decl_var(
        self, ctx: ASLParser.Timestamp_path_decl_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timestamp_path_decl_value.
    def visitTimestamp_path_decl_value(
        self, ctx: ASLParser.Timestamp_path_decl_valueContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_array.
    def visitItems_array(self, ctx: ASLParser.Items_arrayContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_jsonata.
    def visitItems_jsonata(self, ctx: ASLParser.Items_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_path_decl_path_context_object.
    def visitItems_path_decl_path_context_object(
        self, ctx: ASLParser.Items_path_decl_path_context_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_path_decl_path_var.
    def visitItems_path_decl_path_var(
        self, ctx: ASLParser.Items_path_decl_path_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_path_decl_path.
    def visitItems_path_decl_path(self, ctx: ASLParser.Items_path_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_concurrency_jsonata.
    def visitMax_concurrency_jsonata(
        self, ctx: ASLParser.Max_concurrency_jsonataContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_concurrency_int.
    def visitMax_concurrency_int(self, ctx: ASLParser.Max_concurrency_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_concurrency_path_var.
    def visitMax_concurrency_path_var(
        self, ctx: ASLParser.Max_concurrency_path_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_concurrency_path.
    def visitMax_concurrency_path(self, ctx: ASLParser.Max_concurrency_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#parameters_decl.
    def visitParameters_decl(self, ctx: ASLParser.Parameters_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#credentials_decl.
    def visitCredentials_decl(self, ctx: ASLParser.Credentials_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_jsonata.
    def visitRole_arn_jsonata(self, ctx: ASLParser.Role_arn_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_path.
    def visitRole_arn_path(self, ctx: ASLParser.Role_arn_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_path_context_obj.
    def visitRole_arn_path_context_obj(
        self, ctx: ASLParser.Role_arn_path_context_objContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_intrinsic_func.
    def visitRole_arn_intrinsic_func(
        self, ctx: ASLParser.Role_arn_intrinsic_funcContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_var.
    def visitRole_arn_var(self, ctx: ASLParser.Role_arn_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#role_arn_str.
    def visitRole_arn_str(self, ctx: ASLParser.Role_arn_strContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timeout_seconds_jsonata.
    def visitTimeout_seconds_jsonata(
        self, ctx: ASLParser.Timeout_seconds_jsonataContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timeout_seconds_int.
    def visitTimeout_seconds_int(self, ctx: ASLParser.Timeout_seconds_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timeout_seconds_path_decl_var.
    def visitTimeout_seconds_path_decl_var(
        self, ctx: ASLParser.Timeout_seconds_path_decl_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#timeout_seconds_path_decl_path.
    def visitTimeout_seconds_path_decl_path(
        self, ctx: ASLParser.Timeout_seconds_path_decl_pathContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#heartbeat_seconds_jsonata.
    def visitHeartbeat_seconds_jsonata(
        self, ctx: ASLParser.Heartbeat_seconds_jsonataContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#heartbeat_seconds_int.
    def visitHeartbeat_seconds_int(self, ctx: ASLParser.Heartbeat_seconds_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#heartbeat_seconds_path_decl_var.
    def visitHeartbeat_seconds_path_decl_var(
        self, ctx: ASLParser.Heartbeat_seconds_path_decl_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#heartbeat_seconds_path_decl_path.
    def visitHeartbeat_seconds_path_decl_path(
        self, ctx: ASLParser.Heartbeat_seconds_path_decl_pathContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#variable_sample.
    def visitVariable_sample(self, ctx: ASLParser.Variable_sampleContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_tmpl_decl.
    def visitPayload_tmpl_decl(self, ctx: ASLParser.Payload_tmpl_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_binding_path.
    def visitPayload_binding_path(self, ctx: ASLParser.Payload_binding_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_binding_path_context_obj.
    def visitPayload_binding_path_context_obj(
        self, ctx: ASLParser.Payload_binding_path_context_objContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_binding_intrinsic_func.
    def visitPayload_binding_intrinsic_func(
        self, ctx: ASLParser.Payload_binding_intrinsic_funcContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_binding_var.
    def visitPayload_binding_var(self, ctx: ASLParser.Payload_binding_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_binding_value.
    def visitPayload_binding_value(self, ctx: ASLParser.Payload_binding_valueContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_arr_decl.
    def visitPayload_arr_decl(self, ctx: ASLParser.Payload_arr_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_decl.
    def visitPayload_value_decl(self, ctx: ASLParser.Payload_value_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_float.
    def visitPayload_value_float(self, ctx: ASLParser.Payload_value_floatContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_int.
    def visitPayload_value_int(self, ctx: ASLParser.Payload_value_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_bool.
    def visitPayload_value_bool(self, ctx: ASLParser.Payload_value_boolContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_null.
    def visitPayload_value_null(self, ctx: ASLParser.Payload_value_nullContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#payload_value_str.
    def visitPayload_value_str(self, ctx: ASLParser.Payload_value_strContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_decl.
    def visitAssign_decl(self, ctx: ASLParser.Assign_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_decl_body.
    def visitAssign_decl_body(self, ctx: ASLParser.Assign_decl_bodyContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_decl_binding.
    def visitAssign_decl_binding(self, ctx: ASLParser.Assign_decl_bindingContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_object.
    def visitAssign_template_value_object(
        self, ctx: ASLParser.Assign_template_value_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_binding_path.
    def visitAssign_template_binding_path(
        self, ctx: ASLParser.Assign_template_binding_pathContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_binding_path_context.
    def visitAssign_template_binding_path_context(
        self, ctx: ASLParser.Assign_template_binding_path_contextContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_binding_var.
    def visitAssign_template_binding_var(
        self, ctx: ASLParser.Assign_template_binding_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_binding_intrinsic_func.
    def visitAssign_template_binding_intrinsic_func(
        self, ctx: ASLParser.Assign_template_binding_intrinsic_funcContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_binding_assign_value.
    def visitAssign_template_binding_assign_value(
        self, ctx: ASLParser.Assign_template_binding_assign_valueContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value.
    def visitAssign_template_value(self, ctx: ASLParser.Assign_template_valueContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_array.
    def visitAssign_template_value_array(
        self, ctx: ASLParser.Assign_template_value_arrayContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_float.
    def visitAssign_template_value_terminal_float(
        self, ctx: ASLParser.Assign_template_value_terminal_floatContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_int.
    def visitAssign_template_value_terminal_int(
        self, ctx: ASLParser.Assign_template_value_terminal_intContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_bool.
    def visitAssign_template_value_terminal_bool(
        self, ctx: ASLParser.Assign_template_value_terminal_boolContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_null.
    def visitAssign_template_value_terminal_null(
        self, ctx: ASLParser.Assign_template_value_terminal_nullContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_expression.
    def visitAssign_template_value_terminal_expression(
        self, ctx: ASLParser.Assign_template_value_terminal_expressionContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#assign_template_value_terminal_str.
    def visitAssign_template_value_terminal_str(
        self, ctx: ASLParser.Assign_template_value_terminal_strContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#arguments_object.
    def visitArguments_object(self, ctx: ASLParser.Arguments_objectContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#arguments_expr.
    def visitArguments_expr(self, ctx: ASLParser.Arguments_exprContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#output_decl.
    def visitOutput_decl(self, ctx: ASLParser.Output_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_object.
    def visitJsonata_template_value_object(
        self, ctx: ASLParser.Jsonata_template_value_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_binding.
    def visitJsonata_template_binding(
        self, ctx: ASLParser.Jsonata_template_bindingContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value.
    def visitJsonata_template_value(self, ctx: ASLParser.Jsonata_template_valueContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_array.
    def visitJsonata_template_value_array(
        self, ctx: ASLParser.Jsonata_template_value_arrayContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_float.
    def visitJsonata_template_value_terminal_float(
        self, ctx: ASLParser.Jsonata_template_value_terminal_floatContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_int.
    def visitJsonata_template_value_terminal_int(
        self, ctx: ASLParser.Jsonata_template_value_terminal_intContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_bool.
    def visitJsonata_template_value_terminal_bool(
        self, ctx: ASLParser.Jsonata_template_value_terminal_boolContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_null.
    def visitJsonata_template_value_terminal_null(
        self, ctx: ASLParser.Jsonata_template_value_terminal_nullContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_expression.
    def visitJsonata_template_value_terminal_expression(
        self, ctx: ASLParser.Jsonata_template_value_terminal_expressionContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jsonata_template_value_terminal_str.
    def visitJsonata_template_value_terminal_str(
        self, ctx: ASLParser.Jsonata_template_value_terminal_strContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#result_selector_decl.
    def visitResult_selector_decl(self, ctx: ASLParser.Result_selector_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#state_type.
    def visitState_type(self, ctx: ASLParser.State_typeContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#choices_decl.
    def visitChoices_decl(self, ctx: ASLParser.Choices_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#choice_rule_comparison_variable.
    def visitChoice_rule_comparison_variable(
        self, ctx: ASLParser.Choice_rule_comparison_variableContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#choice_rule_comparison_composite.
    def visitChoice_rule_comparison_composite(
        self, ctx: ASLParser.Choice_rule_comparison_compositeContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_variable_stmt.
    def visitComparison_variable_stmt(
        self, ctx: ASLParser.Comparison_variable_stmtContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_composite_stmt.
    def visitComparison_composite_stmt(
        self, ctx: ASLParser.Comparison_composite_stmtContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_composite.
    def visitComparison_composite(self, ctx: ASLParser.Comparison_compositeContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#variable_decl_path.
    def visitVariable_decl_path(self, ctx: ASLParser.Variable_decl_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#variable_decl_var.
    def visitVariable_decl_var(self, ctx: ASLParser.Variable_decl_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#variable_decl_path_context_object.
    def visitVariable_decl_path_context_object(
        self, ctx: ASLParser.Variable_decl_path_context_objectContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#condition_lit.
    def visitCondition_lit(self, ctx: ASLParser.Condition_litContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#condition_expr.
    def visitCondition_expr(self, ctx: ASLParser.Condition_exprContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_func_var.
    def visitComparison_func_var(self, ctx: ASLParser.Comparison_func_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_func_value.
    def visitComparison_func_value(self, ctx: ASLParser.Comparison_func_valueContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#branches_decl.
    def visitBranches_decl(self, ctx: ASLParser.Branches_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#item_processor_decl.
    def visitItem_processor_decl(self, ctx: ASLParser.Item_processor_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#item_processor_item.
    def visitItem_processor_item(self, ctx: ASLParser.Item_processor_itemContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#processor_config_decl.
    def visitProcessor_config_decl(self, ctx: ASLParser.Processor_config_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#processor_config_field.
    def visitProcessor_config_field(self, ctx: ASLParser.Processor_config_fieldContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#mode_decl.
    def visitMode_decl(self, ctx: ASLParser.Mode_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#mode_type.
    def visitMode_type(self, ctx: ASLParser.Mode_typeContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#execution_decl.
    def visitExecution_decl(self, ctx: ASLParser.Execution_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#execution_type.
    def visitExecution_type(self, ctx: ASLParser.Execution_typeContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#iterator_decl.
    def visitIterator_decl(self, ctx: ASLParser.Iterator_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#iterator_decl_item.
    def visitIterator_decl_item(self, ctx: ASLParser.Iterator_decl_itemContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#item_selector_decl.
    def visitItem_selector_decl(self, ctx: ASLParser.Item_selector_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#item_reader_decl.
    def visitItem_reader_decl(self, ctx: ASLParser.Item_reader_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#items_reader_field.
    def visitItems_reader_field(self, ctx: ASLParser.Items_reader_fieldContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#reader_config_decl.
    def visitReader_config_decl(self, ctx: ASLParser.Reader_config_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#reader_config_field.
    def visitReader_config_field(self, ctx: ASLParser.Reader_config_fieldContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#input_type_decl.
    def visitInput_type_decl(self, ctx: ASLParser.Input_type_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#csv_header_location_decl.
    def visitCsv_header_location_decl(
        self, ctx: ASLParser.Csv_header_location_declContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#csv_headers_decl.
    def visitCsv_headers_decl(self, ctx: ASLParser.Csv_headers_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_items_jsonata.
    def visitMax_items_jsonata(self, ctx: ASLParser.Max_items_jsonataContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_items_int.
    def visitMax_items_int(self, ctx: ASLParser.Max_items_intContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_items_path_var.
    def visitMax_items_path_var(self, ctx: ASLParser.Max_items_path_varContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_items_path.
    def visitMax_items_path(self, ctx: ASLParser.Max_items_pathContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_count_jsonata.
    def visitTolerated_failure_count_jsonata(
        self, ctx: ASLParser.Tolerated_failure_count_jsonataContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_count_int.
    def visitTolerated_failure_count_int(
        self, ctx: ASLParser.Tolerated_failure_count_intContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_count_path_var.
    def visitTolerated_failure_count_path_var(
        self, ctx: ASLParser.Tolerated_failure_count_path_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_count_path.
    def visitTolerated_failure_count_path(
        self, ctx: ASLParser.Tolerated_failure_count_pathContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_percentage_jsonata.
    def visitTolerated_failure_percentage_jsonata(
        self, ctx: ASLParser.Tolerated_failure_percentage_jsonataContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_percentage_number.
    def visitTolerated_failure_percentage_number(
        self, ctx: ASLParser.Tolerated_failure_percentage_numberContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_percentage_path_var.
    def visitTolerated_failure_percentage_path_var(
        self, ctx: ASLParser.Tolerated_failure_percentage_path_varContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#tolerated_failure_percentage_path.
    def visitTolerated_failure_percentage_path(
        self, ctx: ASLParser.Tolerated_failure_percentage_pathContext
    ):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#label_decl.
    def visitLabel_decl(self, ctx: ASLParser.Label_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#result_writer_decl.
    def visitResult_writer_decl(self, ctx: ASLParser.Result_writer_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#result_writer_field.
    def visitResult_writer_field(self, ctx: ASLParser.Result_writer_fieldContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#retry_decl.
    def visitRetry_decl(self, ctx: ASLParser.Retry_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#retrier_decl.
    def visitRetrier_decl(self, ctx: ASLParser.Retrier_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#retrier_stmt.
    def visitRetrier_stmt(self, ctx: ASLParser.Retrier_stmtContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_equals_decl.
    def visitError_equals_decl(self, ctx: ASLParser.Error_equals_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#interval_seconds_decl.
    def visitInterval_seconds_decl(self, ctx: ASLParser.Interval_seconds_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_attempts_decl.
    def visitMax_attempts_decl(self, ctx: ASLParser.Max_attempts_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#backoff_rate_decl.
    def visitBackoff_rate_decl(self, ctx: ASLParser.Backoff_rate_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#max_delay_seconds_decl.
    def visitMax_delay_seconds_decl(self, ctx: ASLParser.Max_delay_seconds_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#jitter_strategy_decl.
    def visitJitter_strategy_decl(self, ctx: ASLParser.Jitter_strategy_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#catch_decl.
    def visitCatch_decl(self, ctx: ASLParser.Catch_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#catcher_decl.
    def visitCatcher_decl(self, ctx: ASLParser.Catcher_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#catcher_stmt.
    def visitCatcher_stmt(self, ctx: ASLParser.Catcher_stmtContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#comparison_op.
    def visitComparison_op(self, ctx: ASLParser.Comparison_opContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#choice_operator.
    def visitChoice_operator(self, ctx: ASLParser.Choice_operatorContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#states_error_name.
    def visitStates_error_name(self, ctx: ASLParser.States_error_nameContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#error_name.
    def visitError_name(self, ctx: ASLParser.Error_nameContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#json_obj_decl.
    def visitJson_obj_decl(self, ctx: ASLParser.Json_obj_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#json_binding.
    def visitJson_binding(self, ctx: ASLParser.Json_bindingContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#json_arr_decl.
    def visitJson_arr_decl(self, ctx: ASLParser.Json_arr_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#json_value_decl.
    def visitJson_value_decl(self, ctx: ASLParser.Json_value_declContext):
        return self.visitChildren(ctx)

    # Visit a parse tree produced by ASLParser#keyword_or_string.
    def visitKeyword_or_string(self, ctx: ASLParser.Keyword_or_stringContext):
        return self.visitChildren(ctx)


del ASLParser
